/* $Id$ */

#include "image/imageException.hpp"
#include "image/cImage.h"
#include "cImagePrivate.h"

int JfrImage_colorSpaceChannels(JfrImage_TypeColorSpace cs) {
  switch(cs) {
    case JfrImage_CS_GRAY:
    case JfrImage_CS_BayerBG:
    case JfrImage_CS_BayerRG:
    case JfrImage_CS_BayerGB:
    case JfrImage_CS_BayerGR:
      return 1;
    case JfrImage_CS_BGR:
    case JfrImage_CS_RGB:
    case JfrImage_CS_HSV:
    case JfrImage_CS_HSL:
    case JfrImage_CS_CMY:
    case JfrImage_CS_CMYK:
    case JfrImage_CS_LUV:
    case JfrImage_CS_LAB:
    case JfrImage_CS_XYZ:
    case JfrImage_CS_YXY:
    case JfrImage_CS_YCbCr:
      return 3;
    default:
    case JfrImage_CS_UNKNOWN:
      return 0;
  }
}

int channelDepthSize(int depth)
{
	switch (depth)
	{
		case CV_8U: case CV_8S: return 1;
		case CV_16U: case CV_16S: return 2;
		case CV_32S: case CV_32F: return 4;
		case CV_64F: return 8;
		default: return 0;
	}
}

static int JfrImage_convertColorMode(JfrImage_TypeColorSpace from, JfrImage_TypeColorSpace to)
{
  if (from == JfrImage_CS_GRAY && to == JfrImage_CS_BGR)
    return CV_GRAY2BGR;
  else if (from == JfrImage_CS_GRAY && to == JfrImage_CS_RGB)
    return CV_GRAY2RGB;
  else if (from == JfrImage_CS_BGR && to == JfrImage_CS_GRAY)
    return CV_BGR2GRAY;
  else if (from == JfrImage_CS_RGB && to == JfrImage_CS_GRAY)
    return CV_RGB2GRAY;
  else if (from == JfrImage_CS_BGR && to == JfrImage_CS_RGB)
    return CV_BGR2RGB;
  else if (from == JfrImage_CS_RGB && to == JfrImage_CS_BGR)
    return CV_RGB2BGR;
  else if (from == JfrImage_CS_BayerBG && to == JfrImage_CS_BGR)
    return CV_BayerBG2BGR;
  else if (from == JfrImage_CS_BayerRG && to == JfrImage_CS_BGR)
    return CV_BayerRG2BGR;
  else if (from == JfrImage_CS_BayerGB && to == JfrImage_CS_BGR)
    return CV_BayerGB2BGR;
  else if (from == JfrImage_CS_BayerGR && to == JfrImage_CS_BGR)
    return CV_BayerGR2BGR;
  else if (from == JfrImage_CS_LUV && to == JfrImage_CS_RGB)
    return CV_Luv2RGB;
  else if (from == JfrImage_CS_RGB && to == JfrImage_CS_LUV)
    return CV_RGB2Luv;
  else if (from == JfrImage_CS_LUV && to == JfrImage_CS_BGR)
    return CV_Luv2BGR;
  else if (from == JfrImage_CS_BGR && to == JfrImage_CS_LUV)
    return CV_BGR2Luv;
  else if (from == JfrImage_CS_YCbCr && to == JfrImage_CS_RGB)
    return CV_YCrCb2RGB;
  else if (from == JfrImage_CS_RGB && to == JfrImage_CS_YCbCr)
    return CV_RGB2YCrCb;
  else if (from == JfrImage_CS_YCbCr && to == JfrImage_CS_BGR)
    return CV_YCrCb2BGR;
  else if (from == JfrImage_CS_BGR && to == JfrImage_CS_YCbCr)
    return CV_BGR2YCrCb;
  else
    JFR_ERROR(jafar::image::ImageException, 
	      jafar::image::ImageException::INVALID_COLORSPACE_CONVERSION,
	      "from: " << from << " to: " << to);
}

void JfrImage_convertColorMode(const JfrImage_CImage* src, JfrImage_CImage* dst) {
  try {
    JfrImage_convertColorModePrivate(&src->header, &src->iplImage, &dst->header, &dst->iplImage);
  }
  catch(jafar::image::ImageException& e) {
    JFR_WARNING(e << "conversion aborted");
    return;
  }
}

void JfrImage_convertColorModePrivate(const JfrImage_ImageHeader* jfrSrc, const IplImage* iplSrc,
				      JfrImage_ImageHeader* jfrDst, IplImage* iplDst)
{
  JFR_PRECOND(iplSrc->depth == iplDst->depth,
	      "convertColorMode: invalid depth");
  JFR_PRECOND(iplSrc->width == iplDst->width && iplSrc->height == iplDst->height,
              "convertColorMode: invalid size");

  cvCvtColor(iplSrc, iplDst, JfrImage_convertColorMode(jfrSrc->colorSpace, jfrDst->colorSpace));
}

void JfrImage_convertColorModePrivate(const JfrImage_ImageHeader* jfrSrc, const cv::Mat& src,
				      JfrImage_ImageHeader* jfrDst, cv::Mat& dst)
{
  JFR_PRECOND(src.depth() == dst.depth(),
	      "convertColorMode: invalid depth");
  JFR_PRECOND(src.cols == dst.cols && src.rows == dst.rows,
              "convertColorMode: invalid size");

  cv::cvtColor(src, dst, JfrImage_convertColorMode(jfrSrc->colorSpace, jfrDst->colorSpace));
}

