/* $Id$ */

// boost unit test includes
#define BOOST_TEST_MAIN 
#define BOOST_TEST_DYN_LINK 
#include <boost/test/auto_unit_test.hpp>
using boost::unit_test_framework::test_suite;
using boost::unit_test_framework::test_case;

// jafar debug include
#include "kernel/jafarDebug.hpp"
#include "image/Image.hpp"

using namespace jafar;

BOOST_AUTO_TEST_CASE( roi )
{
	image::Image im1(11, 11, CV_8U, JfrImage_CS_GRAY);
	for(int i = 0; i < im1.height(); ++i)
		for(int j = 0; j < im1.width(); ++j)
			im1.data(i)[j] = 0;
	unsigned char *data = im1.data(0);
	cv::Rect roi;
	cv::Size sz;
	
	im1.setROI(1,2,9,8);
	roi = im1.getROI(sz);
	BOOST_CHECK_EQUAL(roi.x, 1);
	BOOST_CHECK_EQUAL(roi.y, 2);
	BOOST_CHECK_EQUAL(roi.width, 9);
	BOOST_CHECK_EQUAL(roi.height, 8);
	BOOST_CHECK_EQUAL(sz.width, 11);
	BOOST_CHECK_EQUAL(sz.height, 11);
	BOOST_CHECK_EQUAL(im1.data(0), data+roi.y*11+roi.x);
	
	im1.setROI(cv::Rect(2,3,7,6));
	roi = im1.getROI(sz);
	BOOST_CHECK_EQUAL(roi.x, 2);
	BOOST_CHECK_EQUAL(roi.y, 3);
	BOOST_CHECK_EQUAL(roi.width, 7);
	BOOST_CHECK_EQUAL(roi.height, 6);
	BOOST_CHECK_EQUAL(sz.width, 11);
	BOOST_CHECK_EQUAL(sz.height, 11);
	BOOST_CHECK_EQUAL(im1.data(0), data+roi.y*11+roi.x);
	
	image::Image im2(im1, cv::Rect(1,1,6,5));
	roi = im2.getROI(sz);
	BOOST_CHECK_EQUAL(roi.x, 3);
	BOOST_CHECK_EQUAL(roi.y, 4);
	BOOST_CHECK_EQUAL(roi.width, 6);
	BOOST_CHECK_EQUAL(roi.height, 5);
	BOOST_CHECK_EQUAL(sz.width, 11);
	BOOST_CHECK_EQUAL(sz.height, 11);
	BOOST_CHECK_EQUAL(im2.data(0), data+roi.y*11+roi.x);
	
	im2.resetROI();
	roi = im2.getROI(sz);
	BOOST_CHECK_EQUAL(roi.x, 0);
	BOOST_CHECK_EQUAL(roi.y, 0);
	BOOST_CHECK_EQUAL(roi.width, 11);
	BOOST_CHECK_EQUAL(roi.height, 11);
	BOOST_CHECK_EQUAL(sz.width, 11);
	BOOST_CHECK_EQUAL(sz.height, 11);
	BOOST_CHECK_EQUAL(im2.data(0), data+roi.y*11+roi.x);
	
	// check how to modify ROI of a const image
	const image::Image im3(im2, cv::Rect(1,1,9,9));
	roi = im3.getROI(sz);
	BOOST_CHECK_EQUAL(roi.x, 1);
	BOOST_CHECK_EQUAL(roi.y, 1);
	BOOST_CHECK_EQUAL(roi.width, 9);
	BOOST_CHECK_EQUAL(roi.height, 9);
	BOOST_CHECK_EQUAL(sz.width, 11);
	BOOST_CHECK_EQUAL(sz.height, 11);
	//im3.setROI(2,2,7,7); // not possible because im3 is const
	//image::Image im4(im3, cv::Rect(-1,-1,11,11)); // not possible because cannot make the roi bigger with this constructor
	image::Image im4(im3);
	roi = im4.getROI(sz);
	BOOST_CHECK_EQUAL(roi.x, 1);
	BOOST_CHECK_EQUAL(roi.y, 1);
	BOOST_CHECK_EQUAL(roi.width, 9);
	BOOST_CHECK_EQUAL(roi.height, 9);
	BOOST_CHECK_EQUAL(sz.width, 11);
	BOOST_CHECK_EQUAL(sz.height, 11);
	im4.setROI(0,0,11,11);
	roi = im4.getROI(sz);
	BOOST_CHECK_EQUAL(roi.x, 0);
	BOOST_CHECK_EQUAL(roi.y, 0);
	BOOST_CHECK_EQUAL(roi.width, 11);
	BOOST_CHECK_EQUAL(roi.height, 11);
	BOOST_CHECK_EQUAL(sz.width, 11);
	BOOST_CHECK_EQUAL(sz.height, 11);

	// check that no reallocation were done
	im2.data(5)[4] = 42;
	im1.setPixelValue<uchar>(43,3,2);
	BOOST_CHECK_EQUAL(im2.data(5)[5], 43);
	BOOST_CHECK_EQUAL(im1.getPixelValue<uchar>(2,2), 42);
	BOOST_CHECK_EQUAL(im4.data(5)[5], 43);
	
	// check roi overflow management
	im2.setROI(3,-2,20,7);
	roi = im2.getROI();
	BOOST_CHECK_EQUAL(roi.x, 3);
	BOOST_CHECK_EQUAL(roi.y, 0);
	BOOST_CHECK_EQUAL(roi.width, 8);
	BOOST_CHECK_EQUAL(roi.height, 5);
}


BOOST_AUTO_TEST_CASE( robustCopy )
{
	image::Image im1(10, 10, CV_8U, JfrImage_CS_GRAY);
	image::Image im2(10, 10, CV_8U, JfrImage_CS_GRAY);
	
std::cout << __FILE__ << ":" << __LINE__ << std::endl;
	im1.robustCopy(im2, 0,0,0,0,10,10); // nothing to do
	im1.robustCopy(im2, 2,2,3,3,5,5); // nothing to do
std::cout << __FILE__ << ":" << __LINE__ << std::endl;
	
	im1.robustCopy(im2, -2,2,2,2,5,5); // src left
	im1.robustCopy(im2, 2,-2,2,2,5,5); // src top
	im1.robustCopy(im2, 7,2,2,2,5,5); // src right
	im1.robustCopy(im2, 2,7,2,2,5,5); // src bottom
std::cout << __FILE__ << ":" << __LINE__ << std::endl;

	im1.robustCopy(im2, 2,2,-2,2,5,5); // dst left
	im1.robustCopy(im2, 2,2,2,-2,5,5); // dst top
	im1.robustCopy(im2, 2,2,7,2,5,5); // dst right
	im1.robustCopy(im2, 2,2,2,7,5,5); // dst bottom
std::cout << __FILE__ << ":" << __LINE__ << std::endl;
	
	im1.robustCopy(im2, -2,-2,2,2,5,5); // src left+top
	im1.robustCopy(im2, 7,7,2,2,5,5); // src bottom+right
	im1.robustCopy(im2, -2,2,2,2,15,5); // src left+right
	im1.robustCopy(im2, 2,-2,2,2,5,15); // src top+bottom
std::cout << __FILE__ << ":" << __LINE__ << std::endl;

	im1.robustCopy(im2, 2,2,-2,-2,5,5); // dst left+top
	im1.robustCopy(im2, 2,2,7,7,5,5); // dst bottom+right
	im1.robustCopy(im2, 2,2,-2,2,15,5); // dst left+right
	im1.robustCopy(im2, 2,2,2,-2,5,15); // dst top+bottom
std::cout << __FILE__ << ":" << __LINE__ << std::endl;
	
	im1.robustCopy(im2, -2,2,-2,2,5,5); // src+dst left
	im1.robustCopy(im2, 2,-2,2,-2,5,5); // src+dst top
	im1.robustCopy(im2, 7,2,7,2,5,5); // src+dst right
	im1.robustCopy(im2, 2,7,2,7,5,5); // src+dst bottom
std::cout << __FILE__ << ":" << __LINE__ << std::endl;
	
	im1.robustCopy(im2, -2,-2,-2,-2,5,5); // src+dst left+top
	im1.robustCopy(im2, 7,7,7,7,5,5); // src+dst bottom+right
	im1.robustCopy(im2, -2,2,2,2,5,15); // 
	im1.robustCopy(im2, 2,-2,7,2,5,5); // 
	im1.robustCopy(im2, -2,2,2,2,15,5); // 
	im1.robustCopy(im2, -2,7,2,2,5,5); // 
	im1.robustCopy(im2, 7,2,-2,2,15,5); // 
std::cout << __FILE__ << ":" << __LINE__ << std::endl;
}


/*
 * standard init_unit_test_suite functione
 */

test_suite*
init_unit_test_suite( int, char* [] ) {

  // we set the debug level to Warning
  jafar::debug::DebugStream::setDefaultLevel(jafar::debug::DebugStream::Warning);

  return 0;
}

